#ifndef MCDRIVE_H
#define MCDRIVE_H

/*--------------------------------------------------------------
 * class MCDrive
 * offers hich level commands to interact with a servo drive
 *
 * 2020-07-17 AW Frame
 *
 *-------------------------------------------------------------*/
 
//--- inlcudes ----
 
#include <MCTimer.h>
#include <MCNode.h>
#include <stdint.h>

//--- service define ---


typedef enum DriveCommStates {
	eMCIdle,
	eMCWaiting,
	eMCBusy,
	eMCDone,
	eMCError,
	eMCTimeout,
}
 DriveCommStates;

	
class MCDrive {
	public:
		MCDrive();
		void Connect2MsgHandler(MsgHandler *);
		void Connect2MCTimer(MCTimer *);
		void SetNodeId(uint8_t);
		void SetActTime(uint32_t);

		DriveCommStates CheckComState();
		void ResetComState(); 
		void SetTORetryMax(uint8_t);
		void SetBusyRetryMax(uint8_t);
		
		DriveCommStates UpdateDriveStatus();		
		DriveCommStates SendReset();

		DriveCommStates EnableDrive();
		DriveCommStates DisableDrive();
		DriveCommStates StopDrive();


		DriveCommStates SetOpMode(int8_t);
		DriveCommStates SetProfile(uint32_t, uint32_t, uint32_t, int16_t);		
		
		DriveCommStates StartAbsMove(int32_t, bool);
		DriveCommStates StartRelMove(int32_t, bool);
		DriveCommStates ConfigureHoming(int8_t);
		DriveCommStates StartHoming();
		DriveCommStates MoveAtSpeed(int32_t);
		DriveCommStates IsInPos();
		DriveCommStates IsHomingFinished();
		
		CWCommStates GetNodeState();
		SDOCommStates GetSDOState();
		
		uint8_t GetAccessStep();
		
		uint16_t GetSW();
		CWCommStates GetCWAccess();
		
		bool IsLive();
		uint16_t GetLastError();
		
		//hander to be registered at the OsTimer
		static void OnTimeOutCb(void *p) {
			((MCDrive *)p)->OnTimeOut();
		};
	
		MCNode ThisNode;

	private:
		void OnTimeOut();
		DriveCommStates Wait4Status(uint16_t, uint16_t);
		DriveCommStates MovePP(int32_t,bool, bool);
		
		DriveCommStates RxTxState = eMCIdle;
		
		uint8_t AccessStep = 0;
		
		int8_t OpModeRequested;
		int8_t OpModeReported;		
		
		SDOCommStates SDOAccessState = eIdle;
		CWCommStates CWAccessState = eCWIdle;

		uint8_t TORetryCounter = 0;
		uint8_t TORetryMax = 1;
		uint8_t BusyRetryCounter = 0;
		uint8_t BusyRetryMax = 1;
		
		MCTimer *Timer;
		int TimerHandle = 0;
		bool isTimerActive = false;

		uint32_t actTime;

		bool isLive = false;
};

#endif
